using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace ExcelLike
{
    public partial class SimpleSpread : Form
    {
        private const int ROWS_NUMBER = 15;
        private bool[,] _specialCells;
        private int[,] _cells;

        public SimpleSpread()
        {
            InitializeComponent();
        }

        private void SimpleSpread_Load(object sender, EventArgs e)
        {
            DataGridViewCellStyle stl = this._dgv.DefaultCellStyle.Clone();
            stl.Alignment = DataGridViewContentAlignment.MiddleCenter;
            stl.Font = new Font(stl.Font, FontStyle.Bold);
            this._dgv.ColumnHeadersDefaultCellStyle = this._dgv.RowHeadersDefaultCellStyle = stl;
            this._dgv.RowHeadersWidth = 55;
            int start = (int)'A';
            int end = (int)'K';
            for(int i = start; i <= end; i++)
            {
                string name = ((char)i).ToString();
                int index = this._dgv.Columns.Add(name, name);
                this._dgv.Columns[index].SortMode = DataGridViewColumnSortMode.NotSortable;
                this._dgv.Columns[index].Width = 75;
            }
            for(int i = 1; i <= ROWS_NUMBER; i++)
            {
                int index = this._dgv.Rows.Add();
                this._dgv.Rows[index].HeaderCell.Value = i.ToString();
            }
            this._specialCells = new bool[end - start + 1, ROWS_NUMBER];
            this._cells = new int[end - start + 1, ROWS_NUMBER];
        }

        private void _dgv_CellValidating(object sender, DataGridViewCellValidatingEventArgs e)
        {
            this._dgv.Rows[e.RowIndex].ErrorText = "";
            int newInteger;

            if(e.FormattedValue == null || string.IsNullOrEmpty(e.FormattedValue.ToString()) || this._specialCells[e.ColumnIndex, e.RowIndex])
                return;
            if(!int.TryParse(e.FormattedValue.ToString(), out newInteger))
            {
                this._dgv.Rows[e.RowIndex].ErrorText = "Only digits accepted";
                e.Cancel = true;
            }
        }

        private void _tsBtn_MinMax_Click(object sender, EventArgs e)
        {
            int newInteger, count;
            int rowForResult, colForResult, min, max;

            if(this._dgv.SelectedCells.Count <= 1)
            {
                MessageBox.Show("At least 2 cells must be selected", "Error");
                return;
            }

            //clear array
            for(int j = 0; j < this._cells.GetLength(1); j++) //j is rows
                for(int i = 0; i < this._cells.GetLength(0); i++) //i is columns
                    this._cells[i, j] = int.MinValue;

            foreach(DataGridViewCell cell in this._dgv.SelectedCells)
            {
                if(cell.Value != null && int.TryParse(cell.Value.ToString(), out newInteger))
                    this._cells[cell.ColumnIndex, cell.RowIndex] = newInteger;
            }

            //Analyze rows...
            for(int j = 0; j < this._cells.GetLength(1); j++) //j is rows
            {
                count = 0;
                rowForResult = colForResult = 0;
                min = int.MaxValue; max = int.MinValue;
                for(int i = 0; i < this._cells.GetLength(0); i++) //i is columns
                {
                    if(this._cells[i, j] != int.MinValue)
                    {
                        count++;
                        rowForResult = j - (j == this._dgv.Rows.Count ? 1 : 0);
                        colForResult = i + (i < this._dgv.Columns.Count - 1 ? 1 : 0);
                        min = Math.Min(min, this._cells[i, j]); max = Math.Max(max, this._cells[i, j]);
                    }
                }
                if(rowForResult > 0 && colForResult > 0 && count > 1)
                {
                    this._dgv.Rows[rowForResult].Cells[colForResult].Value = string.Format("{0}/{1}", min.ToString(), max.ToString());
                    this._specialCells[colForResult, rowForResult] = true;
                }
            }

            //Analyze columns...
            for(int i = 0; i < this._cells.GetLength(0); i++) //i is columns
            {
                count = 0;
                rowForResult = colForResult = 0;
                min = int.MaxValue; max = int.MinValue;
                for(int j = 0; j < this._cells.GetLength(1); j++) //j is rows
                {
                    if(this._cells[i, j] != int.MinValue)
                    {
                        count++;
                        rowForResult = j + (j < this._dgv.Rows.Count - 1 ? 1 : 0);
                        colForResult = i - (i == this._dgv.Columns.Count ? 1 : 0);
                        min = Math.Min(min, this._cells[i, j]); max = Math.Max(max, this._cells[i, j]);
                    }
                }
                if(rowForResult > 0 && colForResult > 0 && count > 1)
                {
                    this._dgv.Rows[rowForResult].Cells[colForResult].Value = string.Format("{0}/{1}", min.ToString(), max.ToString());
                    this._specialCells[colForResult, rowForResult] = true;
                }
            }
        }

        private void _dgv_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            this._dgv.Rows[e.RowIndex].ErrorText = "";
        }

        private void _dgv_ColumnHeaderMouseClick(object sender, DataGridViewCellMouseEventArgs e)
        {
            if(Control.ModifierKeys != Keys.Control)
                this._dgv.CurrentCell = this._dgv.Rows[0].Cells[e.ColumnIndex];
            for(int i = 0; i < ROWS_NUMBER; i++)
                this._dgv.Rows[i].Cells[e.ColumnIndex].Selected = true;
        }
    }
}